package com.zilber.boot.activiti.controller;


import com.github.pagehelper.PageInfo;
import com.zilber.boot.activiti.dto.*;
import com.zilber.boot.activiti.service.*;
import com.zilber.boot.activiti.vo.DefinitionVO;
import com.zilber.boot.activiti.vo.HistoricTaskVO;
import com.zilber.boot.activiti.vo.InstanceVO;
import com.zilber.boot.activiti.vo.TaskVO;
import com.zilber.boot.utils.AjaxResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;

@Validated
@RestController
@RequestMapping("/bpmn")
@Api(tags = "工作流接口")
@Slf4j
public class BpmnController {

    @Resource
    private DefinitionService definitionService;

    @Resource
    private InstanceService instanceService;

    @Resource
    private TaskService taskService;

    @Resource
    private HistoryService historyService;

    @PostMapping("/definition/upload")
    @ApiOperation("上传流程图")
    public AjaxResult uploadAndDeployment(
            @RequestParam @ApiParam("名称") String name,
            @RequestParam @ApiParam("上传的文件") MultipartFile file) {
        definitionService.uploadAndDeployment(name, file);
        return AjaxResult.success();
    }

    @PostMapping("/definition")
    @ApiOperation("保存流程图")
    public AjaxResult saveAndDeployment(@RequestBody @Valid DeploymentDTO dto) {
        definitionService.saveAndDeployment(dto.getName(), dto.getXml());
        return AjaxResult.success();
    }

    @GetMapping("/definition/page")
    @ApiOperation("查询流程定义列表")
    public AjaxResult getDefinitionPage(@Valid PageQueryDTO dto) {
        PageInfo<DefinitionVO> page = new PageInfo<>(definitionService.getDefinitionPage(dto));
        return AjaxResult.success(page);
    }

    @GetMapping("/definition/list/by/{key}")
    @ApiOperation("查询指定Key的流程定义所有列表")
    public AjaxResult getVersionList(
            @PathVariable @NotBlank String key) {
        return AjaxResult.success(definitionService.getVersionList(key));
    }

    @GetMapping("/definition/by/key/{key}")
    @ApiOperation("查询流程图，XML字符串格式")
    public AjaxResult getDeploymentXMLByKey(
            @PathVariable @ApiParam("流程定义Key") @NotBlank String key) {
        String xmlString = definitionService.getDeploymentXML(key);
        HashMap<String, String> map = new HashMap<>();
        map.put("xml", xmlString);
        return AjaxResult.success(map);
    }

    @GetMapping("/definition/by/{id}")
    @ApiOperation("查询流程图，XML字符串格式")
    public AjaxResult getDeploymentXML(
            @PathVariable @ApiParam("流程部署ID") @NotBlank String id,
            @RequestParam @ApiParam("资源文件名") @NotBlank String name) {
        String xmlString = definitionService.getDeploymentXML(id, name);
        HashMap<String, String> map = new HashMap<>();
        map.put("xml", xmlString);
        return AjaxResult.success(map);
    }

    @GetMapping("/definition/file/{id}")
    @ApiOperation("查询流程图，XML文件格式")
    public void getDeploymentFile(
            HttpServletResponse response,
            @PathVariable @ApiParam("流程部署ID") @NotBlank String id,
            @RequestParam @ApiParam("资源文件名") @NotBlank String name) {
        InputStream is = definitionService.getDeploymentStream(id, name);
        OutputStream os;
        try {
            os = response.getOutputStream();
            IOUtils.copy(is, os);

            is.close();
            os.close();
        } catch (IOException e) {
            log.error("查看流程文件失败", e);
        }
    }

    @DeleteMapping("/definition/by/{id}")
    @ApiOperation("删除流程定义")
    public AjaxResult deleteDefinitionById(
            @PathVariable @ApiParam("流程部署ID") @NotBlank String id) {
        definitionService.deleteById(id);
        return AjaxResult.success();
    }

    @GetMapping("/instance/page")
    @ApiOperation("查询流程实例列表")
    public AjaxResult getInstancePage(@Valid PageQueryDTO dto) {
        PageInfo<InstanceVO> page = new PageInfo<>(instanceService.getInstancePage(dto));
        return AjaxResult.success(page);
    }

    @PostMapping("/instance")
    @ApiOperation("启动流程实例")
    public AjaxResult startInstance(@RequestBody @Valid InstanceDTO dto) {
        instanceService.start(dto);
        return AjaxResult.success();
    }

    @PutMapping("/instance/suspend/by/{id}")
    @ApiOperation("挂起流程实例")
    public AjaxResult suspendInstance(@PathVariable @ApiParam("流程实例ID") @NotBlank String id) {
        instanceService.suspend(id);
        return AjaxResult.success();
    }

    @PutMapping("/instance/resume/by/{id}")
    @ApiOperation("激活/重启流程实例")
    public AjaxResult resumeInstance(@PathVariable @ApiParam("流程实例ID") @NotBlank String id) {
        instanceService.resume(id);
        return AjaxResult.success();
    }

    @DeleteMapping("/instance/by/{id}")
    @ApiOperation("取消流程实例")
    public AjaxResult deleteInstanceById(@PathVariable @ApiParam("流程实例ID") @NotBlank String id) {
        instanceService.cancelById(id);
        return AjaxResult.success();
    }

    @GetMapping("/instance/variables/by/{id}")
    @ApiOperation("查询流程参数")
    public AjaxResult instanceVariables(@PathVariable @ApiParam("流程实例ID") @NotBlank String id) {
        return AjaxResult.success(instanceService.getVariables(id));
    }

    @GetMapping("/task/page")
    @ApiOperation("查看待办任务")
    public AjaxResult taskPage(@Valid TaskQueryDTO dto) {
        PageInfo<TaskVO> page = new PageInfo<>(taskService.taskPage(dto));
        return AjaxResult.success(page);
    }

    @PutMapping("/task/{id}")
    @ApiOperation("完成任务")
    public AjaxResult completeTask(
            @PathVariable @NotBlank @ApiParam("流程任务ID") String id,
            @RequestBody @Valid TaskCompleteDTO dto) {
        taskService.complete(dto.getUserCode(), id, dto);
        return AjaxResult.success();
    }

    @GetMapping("/historic/task/page")
    @ApiOperation("查看历史任务")
    public AjaxResult historicTaskPage(@Valid TaskQueryDTO dto) {
        PageInfo<HistoricTaskVO> page = new PageInfo<>(historyService.historicTaskPage(dto));
        return AjaxResult.success(page);
    }

    @GetMapping("/historic/task/list")
    @ApiOperation("任务实例历史")
    public List<HistoricTaskVO> getHistoryListByInstanceId(
            @RequestParam @ApiParam("流程实例ID") @NotBlank String instanceId) {
        return historyService.getListOfInstance(instanceId);
    }
}
