package com.zilber.boot.activiti.service;

import com.zilber.boot.activiti.dto.PageQueryDTO;
import com.zilber.boot.activiti.vo.DefinitionVO;
import com.zilber.boot.exception.ServiceException;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.ProcessDefinition;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipInputStream;

@Service
public class DefinitionService {

    @Autowired
    private RepositoryService repositoryService;

    /**
     * 上传并部署流程定义
     *
     * @param name 流程定义名称
     * @param file 上传的文件
     */
    public void uploadAndDeployment(String name, MultipartFile file) {
        try {
            String filename = file.getOriginalFilename();
            String extension = FilenameUtils.getExtension(filename);
            InputStream fileInputStream = file.getInputStream();
            if ("zip".equals(extension)) {
                ZipInputStream zip = new ZipInputStream(fileInputStream);
                repositoryService.createDeployment()
                        .addZipInputStream(zip)
                        .name(name)
                        .deploy();
            } else {
                repositoryService.createDeployment()
                        .addInputStream(filename, fileInputStream)
                        .name(name)
                        .deploy();
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public List<DefinitionVO> getDefinitionPage(PageQueryDTO dto) {
        List<ProcessDefinition> list = repositoryService.createProcessDefinitionQuery()
                .latestVersion().listPage(dto.getPageNo() * dto.getPageSize(), dto.getPageSize());
        list.sort((y, x) -> x.getVersion() - y.getVersion());

        return list.stream().map(processDefinition -> {
            DefinitionVO vo = new DefinitionVO();
            BeanUtils.copyProperties(processDefinition, vo);
            return vo;
        }).collect(Collectors.toList());
    }

    /**
     * 根据流程定义Key获取历史版本
     *
     * @param key 流程定义Key
     * @return 历史版本列表
     */
    public List<DefinitionVO> getVersionList(String key) {
        List<ProcessDefinition> list = repositoryService.createProcessDefinitionQuery()
                .processDefinitionKey(key)
                .orderByProcessDefinitionVersion().desc()
                .list();
        return list.stream().map(processDefinition -> {
            DefinitionVO vo = new DefinitionVO();
            BeanUtils.copyProperties(processDefinition, vo);
            return vo;
        }).collect(Collectors.toList());
    }

    /**
     * 在线绘制流程图保存
     *
     * @param name 流程名称
     * @param xml  xml数据
     */
    public void saveAndDeployment(String name, String xml) {
        repositoryService.createDeployment()
                .addString("bpmnjs.bpmn", xml)
                .name(name)
                .deploy();
    }

    /**
     * 获取流程图（流）
     *
     * @param deploymentId 流程部署ID
     * @param resourceName 资源文件名称
     * @return 数据流
     */
    public InputStream getDeploymentStream(String deploymentId, String resourceName) {
        return repositoryService.getResourceAsStream(deploymentId, resourceName);
    }

    /**
     * 获取流程图xml
     *
     * @param definitionKey 流程定义Key
     * @return xml字符串
     */
    public String getDeploymentXML(String definitionKey) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery()
                .processDefinitionKey(definitionKey)
                .latestVersion()
                .singleResult();
        return getDeploymentXML(processDefinition.getDeploymentId(), processDefinition.getResourceName());
    }

    /**
     * 获取流程图xml
     *
     * @param deploymentId 流程部署ID
     * @param resourceName 资源文件名称
     * @return xml字符串
     */
    public String getDeploymentXML(String deploymentId, String resourceName) {
        try {
            InputStream inputStream = repositoryService.getResourceAsStream(deploymentId, resourceName);
            StringWriter writer = new StringWriter();
            IOUtils.copy(inputStream, writer, StandardCharsets.UTF_8.name());
            return writer.toString();
        } catch (IOException e) {
            throw new ServiceException(e.getMessage());
        }
    }

    /**
     * 获取流程定义
     *
     * @param processDefinitionId 流程定义ID
     * @return
     */
    public ProcessDefinition getById(String processDefinitionId) {
        return repositoryService.createProcessDefinitionQuery()
                .processDefinitionId(processDefinitionId)
                .singleResult();
    }

    /**
     * 根据流程定义id删除流程定义
     *
     * @param id 流程定义id
     */
    public void deleteById(String id) {
        repositoryService.deleteDeployment(id);
    }
}
